/*
** Copyright (c) 1992, 1993-2001 by Wayne E. Wright, W5XD, Round Rock, Texas
** All rights reserved.
*/
#include "stdafx.h"
#include "WlogRigs.h"
#include "Rig.h"
#include "RwRig.h"
#include "Kachina.h"
#include "DrvTable.h"

CKachina::CKachina()
{
	m_CurrentState = NONE;
	m_TimerCount = 0;
}



STDMETHODIMP CKachina::ClearRIT()
{
	return S_OK;
}

STDMETHODIMP CKachina::Xmit(/*[in]*/ long State, long *pVal)
{
	static const char XmitOn[] = {'x', 1};
	static const char XmitOff[] = {'x', 0};
	SendString(State ? XmitOn : XmitOff, 2);
	*pVal = RIG_ALL_DONE;
	return S_OK;
}

void CKachina::SendDds(char c, char AB, long f)
{
	char	Buf[16];
	Buf[0] = c;
	long DDS = DdsFromFreq(f);
	Buf[1] = (char)(DDS >> 24);
	Buf[2] = (char)(DDS >> 16);
	Buf[3] = (char)(DDS >> 8);
	Buf[4] = (char) DDS;
	Buf[1] |= AB << 2;
	SendString(Buf, 5);
}

STDMETHODIMP CKachina::Check(long *pVal)
{
	int Max;
	char * GoodSpot;

	*pVal = 10;
	GoodSpot = m_State.rbuf;
	Max = RigRead(&m_State);

	switch (m_CurrentState)
	{
	case SET_1:
		LogMessage("SET_1\n");
		m_State.m_CharsInBuffer = 0;
		while (Max--)
		{
			if (*GoodSpot++ == (char)0xFF)
			{
				//Set rx Frequency
				m_CurrentState = SET_2;
				SendDds('R', 1, (long)(m_LastFreq.Rcv * 100.0));
				break;
			}
		}
		break;

	case SET_2:
		LogMessage("SET_2\n");
		m_State.m_CharsInBuffer = 0;
		while (Max--)
		{
			if (*GoodSpot++ == (char)0xFF)
			{
				//Set tx Frequency
				m_CurrentState = SET_3;
				SendDds('T', 1, (long)(m_LastFreq.Xmit*100.0));
				break;
			}
		}
		break;

	case SET_3:
		LogMessage("SET_3\n");
		m_State.m_CharsInBuffer = 0;
		while (Max--)
		{
			if (*GoodSpot++ == (char)0xFF)
			{
				//Set function
				m_CurrentState = SET_4;
				SendString("F\001", 2);
				break;
			}
		}
		break;

	case SET_4:
		LogMessage("SET_4\n");
		m_State.m_CharsInBuffer = 0;
		while (Max--)
		{
			if (*GoodSpot++ == (char)0xFF)
			{
				*pVal = RIG_ALL_DONE;
				m_CurrentState = NONE;
				break;
			}
		}
		break;


	case CHECK_1:
		LogMessage("check_1\n");
		*pVal = RIG_ALL_DONE;
		m_State.m_CharsInBuffer = 0;
		break;


	default:
		m_State.m_CharsInBuffer = 0;
		break;
	}
	return S_OK;
}

STDMETHODIMP CKachina::Launch(long *pVal)
{
	m_CurrentState = CHECK_1;
	*pVal = 10;
	LogMessage("Launch\n");
	return S_OK;
}

void CKachina::SendString(const char *s, long c)
{
	char	Buffer[256];
	int i = 0;
	Buffer[i++] = 2;
	while (c--)
	{
		Buffer[i++] = *s++;
	}
	Buffer[i++] = 3;
	RigWrite(Buffer, i, m_State.m_pCommChannel);
	m_State.m_CharsInBuffer = 0;
}

STDMETHODIMP CKachina::Set(
						   double Xmit,
						   double Rcv,
						   short SetMode,
						   short PrimaryVFO,
						   short Split,
						   long *pVal)
{
	char Mode[2];
	m_LastFreq.Xmit = Xmit;
	m_LastFreq.Rcv = Rcv;
	m_LastFreq.Mode = SetMode;
	m_LastFreq.PrimaryVFO = PrimaryVFO;
	m_LastFreq.Split = Split;
	m_TimerCount = 0;
	m_State.m_CharsInBuffer = 0;

	//Set Mode First
	Mode[0] = 'M';
	Mode[1] = 0;
	switch(static_cast<WlMode_t>(SetMode))
	{
	case WL_MODE_FSK:	//FSK
	case WL_MODE_LSB:	//LSB
		Mode[1] = 5;
		break;
	case WL_MODE_USB:	//USB
		Mode[1] = 4;
		break;
	case WL_MODE_CW:	//CW
		Mode[1] = 2;
		break;
	case WL_MODE_FM:	//FM
		Mode[1] = 3;
		break;
	case WL_MODE_AM:	//AM
		Mode[1] = 1;
		break;
	}
	SendString(Mode, 2);
	m_CurrentState = SET_1;
	*pVal = 10;
	return S_OK;
}


//IWlRigInfo
STDMETHODIMP CKachina::get_Label( BSTR *pVal)
{
	*pVal = SysAllocString(L"Kachina");
	return S_OK;
}

STDMETHODIMP CKachina::get_FpProgId(BSTR *pVal)
{
	*pVal = SysAllocString(L"WriteLog.DemoFP.1");
	return S_OK;
}

// IKachina
STDMETHODIMP CKachina::put_IF_Filter( long FilterWidth)
{
	return S_OK;
}

STDMETHODIMP CKachina::get_IF_Filter( long  *pFilterWidth)
{
	*pFilterWidth = 400;
	return S_OK;
}
STDMETHODIMP CKachina::put_Split(long s)
{
	return S_OK;
}
STDMETHODIMP CKachina::get_Split(long *pS)
{
	*pS = 0;
	return S_OK;
}


//Class Factory-like stuff

extern "C" HRESULT KachinaFactory(long, IWlRig **pRig)
{
	CComObject<CKachina> *pObject;
	HRESULT hr = pObject->CreateInstance(&pObject);
	if (SUCCEEDED(hr))
	{
		*pRig = pObject;
		pObject->AddRef();
	}
	return hr;
}

static char *label(struct DrvTable *Entry)
{
    return "Kachina 505 fp";
}

static int baud(struct DrvTable *Entry)
{
    return 9600;
}


extern "C" struct DrvTable KachinaTable = 
	{
    (char *(*)())label,
    (int (*)())baud,
	KachinaFactory,
	};
