/*
** Copyright (c) 1992, 1993-2001 by Wayne E. Wright, W5XD, Round Rock, Texas
** All rights reserved.
*/
#ifndef KACHINA_H
#define KACHINA_H
#include "rig.h"
#include "resource.h"       // main symbols
#include "rigdisp.h"
#include "WlogRigLib.h"

/////////////////////////////////////////////////////////////////////////////
// CKachina
//
// This is a partially implemented Kachina driver.
// It only does a couple of functions, but shows the basic
// features necessary for WL to think its a legitimate rig driver
class ATL_NO_VTABLE CKachina : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IWlRigFpInfo,
	public IKachina,
	public CRigBase
{
public:
	CKachina();

DECLARE_NOT_AGGREGATABLE(CKachina)

BEGIN_COM_MAP(CKachina)
	COM_INTERFACE_ENTRY(IWlRig)
	COM_INTERFACE_ENTRY(IWlRigFpInfo)
	COM_INTERFACE_ENTRY(IKachina)
END_COM_MAP()

// IWlRig
public:
	STDMETHOD(ClearRIT)();
	STDMETHOD(Xmit)(/*[in]*/ long State, long *pVal);
	STDMETHOD(Check)(long *pVal);
	STDMETHOD(Launch)(long *pVal);
	STDMETHOD(Set)(
		double Xmit,
		double Rcv,
		short Mode,
		short PrimaryVFO,
		short Split,
		long *pVal);

// IKachina
    STDMETHOD(put_IF_Filter)( long FilterWidth);       
	STDMETHOD(get_IF_Filter)( long  *pFilterWidth);
	STDMETHOD(put_Split)(long Split);
	STDMETHOD(get_Split)(long *pSplit);

// IWlRigFpInfo
public:
	STDMETHOD(get_Label)( BSTR *pVal);
	STDMETHOD(get_FpProgId)(BSTR *pVal);

protected:
	int	m_TimerCount;
	void SendDds(char, char, long);
	long	DdsFromFreq(long freq)
	{
		return (long)(2.2369621333 * (75000000.0 + (freq * 10.0)));
	}
	long    FreqFromDds(long Dds)
	{
		return (long)((((double) Dds / 2.2369621333) - 75000000.0)/10.0);
	}
	void SendString(const char *s, long c);

	~CKachina()
	{
	}

	enum {NONE, SET_1, SET_2, SET_3, SET_4, CHECK_1} m_CurrentState;
};


#endif