/*
** Copyright (c) 1992, 1993 by Wayne E. Wright, W5XD, Round Rock, Texas
** All rights reserved.
*/
/*
**  rigdisp.c
*/

#include "stdafx.h"
#ifdef RIG_DEBUG
#define WRITE_DEBUG_INFO 1
#include <stdio.h>
#else
#define WRITE_DEBUG_INFO 0
#endif

#include "freq.h"
#include "DrvTable.h"
#include "rwrig.h"


#if WRITE_DEBUG_INFO
FILE *IoLogFile;
void LogMessage(const char *v)
{
	if (IoLogFile)
		fputs(v, IoLogFile);
}
#endif

void RigCloseLog()
{
#if WRITE_DEBUG_INFO
	if (IoLogFile)
		fclose(IoLogFile);
	IoLogFile = 0;
#endif
}


void
RigWrite(
    register const char *com,
    int length,
    IWlRigIO *pCommChannel)
   {
	if (length == 0)
		return;
	if (pCommChannel == 0)
		return;		// can't do anything if no COMM port
	VARIANT v;
    long i;
	VariantInit(&v);
	V_VT(&v) = VT_UI1 | VT_ARRAY;
	v.parray = SafeArrayCreateVector(VT_UI1, 0, length);
	void *p;
	if (SUCCEEDED(SafeArrayAccessData(v.parray, &p)))
	{
		memcpy(p, com, length);
		SafeArrayUnaccessData(v.parray);
	}
	pCommChannel->WriteChars(v, &i);
	VariantClear(&v);

#if WRITE_DEBUG_INFO
    if (!IoLogFile)
        IoLogFile = fopen("wlogio.log", "w");
    if (IoLogFile)
    {
    fputs("write: ", IoLogFile);

    if (i)
        fprintf(IoLogFile, "ERROR: %x ", i);
    for (i = 0; i < length; i += 1)
            fprintf(IoLogFile, "%2x ", (unsigned char) com[i]);
    fputs("\n", IoLogFile);
    }
#endif
}

int
RigRead(DrvState_t CurState)
{
    DWORD Max=0;
	long i; VARIANT v;
	long CommError;
	
	// Can't do anything if no interface to COMM port
	if (CurState->m_pCommChannel == 0)
		return 0;

	int CharsInBuffer = CurState->m_CharsInBuffer;
	
	VariantInit(&v);
	CurState->m_pCommChannel->ReadChars(&v, &CommError );

	if (V_VT(&v) == (VT_UI1 | VT_ARRAY))
	{
		void *p;
		long ub, lb;
		SafeArrayGetUBound(v.parray, 1, &ub);
		SafeArrayGetLBound(v.parray, 1, &lb);
		if (SUCCEEDED(SafeArrayAccessData(v.parray, &p)))
		{
			unsigned long c = ub - lb + 1;
			if (c > (sizeof(CurState->rbuf) - CharsInBuffer))
				c = sizeof(CurState->rbuf) - CharsInBuffer;
			memcpy(CurState->rbuf + CharsInBuffer, p, c);
			Max = c;
			SafeArrayUnaccessData(v.parray);
		}
	}

	VariantClear(&v);


    i = 0;
    Max += CharsInBuffer;
	CurState->m_CharsInBuffer = Max;

#if WRITE_DEBUG_INFO
    if (IoLogFile)
    {
      int j;
		j = 0;
      fprintf(IoLogFile, "read(%d, %d): ", CharsInBuffer, Max);
      if (CommError)
        fprintf(IoLogFile, "ERROR %x ", CommError);
      else
        for (i = 0; i < Max; i += 1)
		{
            fprintf(IoLogFile, "%2x ", (unsigned char) CurState->rbuf[i]);
			j += 1;
			if (j > 25)
			{
				fputs("\n", IoLogFile);
				j = 0;
			}
		}
      fputs("\n", IoLogFile);
    }
#endif
    return Max;
}


